=== added file '.bzrignore'
--- .bzrignore	1970-01-01 00:00:00 +0000
+++ .bzrignore	2009-09-30 20:51:34 +0000
@@ -0,0 +1,2 @@
+*.pyc
+*.pyo

=== added file 'README.txt'
--- README.txt	1970-01-01 00:00:00 +0000
+++ README.txt	2009-09-30 20:51:34 +0000
@@ -0,0 +1,63 @@
+=========================================
+  bzr bash-completion script and plugin
+=========================================
+
+This script generates a shell function which can be used by bash to
+automatically complete the currently typed command when the user
+presses the completion key (usually tab).
+
+It can be used either as a bzr plugin or directly.
+
+----------------------------------------
+1. Installing as a plugin
+
+You only need to do this if you want to use the script as a bzr
+plugin.  Otherwise simply grab the bashcomp.py and place it wherever
+you want.
+
+  mkdir -p ~/.bazaar/plugins
+  cd ~/.bazaar/plugins
+  bzr co lp:bzr-bash-completion bash_completion
+
+
+----------------------------------------
+2. Using as a plugin
+
+This is the preferred method of generating initializing the
+completion, as it will ensure proper bzr initialization.
+
+  eval "`bzr bash-completion`"
+
+
+----------------------------------------
+3. Using as a script
+
+As an alternative, if bzrlib is available to python scripts, the
+following invocation should yield the same results without requiring
+you to add a plugin. Might have some issues, though.
+
+  eval "`./bashcomp.py`"
+
+----------------------------------------
+4. License
+
+As this is built upon a bash completion script originally included in
+the bzr source tree, and as the bzr sources are covered by the GPL 2,
+this script here is licensed under these same terms.
+
+If you require a more liberal license, you'll have to contact all
+those who contributed code to this plugin, be it for bash or for
+python.
+
+----------------------------------------
+5. History
+
+The plugin was created by Martin von Gagern in 2009, building on a
+static completion function of very limited scope distributed together
+with bzr.
+
+----------------------------------------
+6. References
+
+https://launchpad.net/bzr-bash-completion
+http://bazaar-vcs.org/

=== added file '__init__.py'
--- __init__.py	1970-01-01 00:00:00 +0000
+++ __init__.py	2009-09-30 20:51:34 +0000
@@ -0,0 +1,18 @@
+from bzrlib.commands import Command, register_command
+
+class cmd_bash_completion(Command):
+    """Generate a shell function for bash command line completion.
+
+    This command generates a shell function which can be used by bash to
+    automatically complete the currently typed command when the user presses
+    the completion key (usually tab).
+    
+    Commonly used like this:
+        eval "`bzr bash-completion`"
+    """
+    def run(self):
+        import sys
+        from bashcomp import bash_completion_function
+        bash_completion_function(sys.stdout)
+
+register_command(cmd_bash_completion)

=== renamed file 'bzr' => 'bashcomp.py'
--- bzr	2009-09-30 20:48:41 +0000
+++ bashcomp.py	2009-09-30 20:51:34 +0000
@@ -1,3 +1,9 @@
+#!/usr/bin/python
+
+from bzrlib import plugin
+from bzrlib import commands
+
+template="""\
 # Programmable completion for the Bazaar-NG bzr command under bash. Source
 # this file (or on some systems add it to ~/.bash_completion and start a new
 # shell) and bash's completion mechanism will know all about bzr's options!
@@ -8,6 +14,7 @@
 
 # Based originally on the svn bash completition script.
 # Customized by Sven Wilhelm/Icecrash.com
+# Adjusted for automatic generation by Martin von Gagern
 
 _bzr ()
 {
@@ -16,7 +23,7 @@
 	COMPREPLY=()
 	cur=${COMP_WORDS[COMP_CWORD]}
 
-	cmds='status diff commit ci checkin move remove log info check ignored'
+	cmds='%(cmds)s'
 
 	if [[ $COMP_CWORD -eq 1 ]] ; then
 		COMPREPLY=( $( compgen -W "$cmds" -- $cur ) )
@@ -33,41 +40,13 @@
 
 	cmdOpts=
 	case ${COMP_WORDS[1]} in
-	status)
-		cmdOpts="--all --show-ids"
-		;;
-	diff)
-		cmdOpts="-r --revision --diff-options"
-		;;
-	commit|ci|checkin)
-		cmdOpts="-r --message -F --file -v --verbose"
-		;;
-	move)
-		cmdOpts=""
-		;;
-	remove)
-		cmdOpts="-v --verbose"
-		;;
-	log)
-		cmdOpts="--forward --timezone -v --verbose --show-ids -r --revision"
-		;;
-	info)
-		cmdOpts=""
-		;;
-	ignored)
-		cmdOpts=""
-		;;
-	check)
-		cmdOpts=""
-		;;
-	help|h|\?)
-		cmdOpts="$cmds $qOpts"
-		;;
+%(cases)s\
 	*)
+		cmdOpts='--help -h'
 		;;
 	esac
 
-	cmdOpts="$cmdOpts --help -h"
+	cmdOpts=" $cmdOpts "
 
 	# take out options already given
 	for (( i=2; i<=$COMP_CWORD-1; ++i )) ; do
@@ -81,14 +60,9 @@
 		cmdOpts=" $cmdOpts "
 		cmdOpts=${cmdOpts/ ${optBase} / }
 
-		# take out alternatives
+		# take out some alternatives
 		case $optBase in
-		-v)              cmdOpts=${cmdOpts/ --verbose / } ;;
-		--verbose)       cmdOpts=${cmdOpts/ -v / } ;;
-		-h)              cmdOpts=${cmdOpts/ --help / } ;;
-		--help)          cmdOpts=${cmdOpts/ -h / } ;;
-		-r)              cmdOpts=${cmdOpts/ --revision / } ;;
-		--revision)      cmdOpts=${cmdOpts/ -r / } ;;
+%(optalt)s\
 		esac
 
 		# skip next option if this one requires a parameter
@@ -102,3 +76,61 @@
 	return 0
 }
 complete -F _bzr -o default bzr
+"""
+
+def bash_completion_function(out):
+    aliases = []
+    cases = ""
+    optaliases = {}
+    reqarg = {}
+    for name in sorted(commands.all_command_names()):
+        cmd = commands.get_cmd_object(name)
+        cases += "\t" + name
+        aliases.append(name)
+        for alias in cmd.aliases:
+            cases += "|" + alias
+            aliases.append(alias)
+        cases += ")\n"
+        plugin = cmd.plugin_name()
+        if plugin is not None:
+            cases += "\t\t# plugin \"%s\"\n" % plugin
+        opts = cmd.options()
+        optnames = []
+        for optname in sorted(cmd.options()):
+            opt = opts[optname]
+            optset = set()
+            for (name, short_name, optname, help) in opt.iter_switches():
+                if short_name is not None:
+                    optset.add("-" + short_name)
+                if name is not None:
+                    optset.add("--" + name)
+            for optname in optset:
+                if optname not in optaliases:
+                    optaliases[optname] = optset
+                else:
+                    optaliases[optname] &= optset
+            optnames.extend(sorted(optset))
+        cases += "\t\tcmdOpts='" + " ".join(optnames) + "'\n\t\t;;\n"
+    optalt = ""
+    for opt1 in sorted(optaliases):
+        optset = optaliases[opt1]
+        if len(optset) == 1:
+            continue
+        optalt += "\t\t" + opt1 + ")\n"
+        for opt2 in sorted(optset):
+            if opt1 != opt2:
+                optalt += "\t\t\tcmdOpts=${cmdOpts/ " + opt2 + " / }\n"
+        optalt += "\t\t\t;;\n"
+    out.write(template % {"cmds": " ".join(aliases),
+                          "cases": cases,
+                          "optalt": optalt})
+
+if __name__ == '__main__':
+
+    import sys
+    import locale
+
+    locale.setlocale(locale.LC_ALL, '')
+    plugin.load_plugins()
+    commands.install_bzr_command_hooks()
+    bash_completion_function(sys.stdout)

